#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#include "misc.h"
#include "logger.h"

// FIXME -- add error checking for invalid parameters, et al
// FIXME -- instrument the rest of these for function profiling

#warning "Ignore the cast errors -- stdtypes.h is designed to guarantee that the ptrint type is safe"

int file_rd ( char *filename, const char *fmt, ... )
  {
     va_list  ap;
     FILE    *f;
     int      res;
     f = fopen ( filename, "r" );
     if ( f == NULL ) return ( -100 );
     va_start(ap, fmt);
     res = vfscanf ( f, fmt, ap );
     va_end(ap);
     fclose ( f );
     return ( res );
  }

int file_wr ( char *filename, const char *fmt, ... )
  {
     va_list  ap;
     FILE    *f;
     int      res;
     f = fopen ( filename, "w" );
     if ( f == NULL ) return ( -100 );
     va_start(ap, fmt);
     res = vfprintf ( f, fmt, ap );
     va_end(ap);
     fclose ( f );
     return ( res );
  }

int file_ap ( char *filename, const char *fmt, ... )
  {
     va_list  ap;
     FILE    *f;
     int      res;
     f = fopen ( filename, "a" );
     if ( f == NULL ) return ( -100 );
     va_start(ap, fmt);
     res = vfprintf ( f, fmt, ap );
     va_end(ap);
     fclose ( f );
     return ( res );
  }

char *mkstr ( char *buf, char *s, ... )
  {
    va_list ap;
    FENTRY ( "format string", s, "s" );
    va_start ( ap, s );
    vsprintf ( buf, s, ap );
    va_end ( ap );
    FEXIT
    return ( buf );
  }

word32 hasher ( char *str ) 
  {
    word32 ret, i, j, k;
    k = strlen(str);
    for ( i = 0, j = 0, ret = 0; i < k; i++ ) 
      {
        word32 x;
        x = str[i] - ' ';
        j += ( ( ( i & 4 ) >> 2 ) + ( i & 1 ) ) >> 1;
        j &= 3;
        ret += x << ( ( ( i & 3 ) << 3 ) + j );
      }
    return ret;
  } // end of function hasher()

//   "name" gives the name of a file, and "paths" is a series of path
//   strings, which must each end in "/" and are separated by ":".
//   try_fopen() will try to open "name" in each of these paths in 
//   sequence, and stop at the first successful opening, returning 
//   the path of the successfully opened file in "dstr" (note: only the 
//   path part is returned in dstr, not the pathname).
// interface
int try_fopen ( char *dstr, FILE **f_, char *name, char *paths_ )
  {
    FILE *f = *f_;
    char *keeper;
    char *paths;
    char *pathname;
    char *apath;
    word32 n;

    paths = strdup ( paths_ );
    keeper = paths;
    n = strlen ( paths_ ) + strlen ( name ) + 1;
    pathname = (char*) malloc ( n );
    apath = strsep ( &paths, ":" );
    while ( apath != NULL )
      {
        sprintf ( pathname, "%s%s", apath, name );
        f = fopen ( pathname, "r" );
        if ( f != NULL )
          {
            strcpy ( dstr, apath );
            free ( keeper );
            free ( pathname );
            *f_ = f;
            return ( 0 );
	  }
      }
    free ( keeper );
    free ( pathname );
    dstr[0] = 0;
    *f_ = NULL;
    return ( -1 );
  }

